@@ -1,10 +1,11 @@
 /*
     ext2_resize.c -- ext2 resizer
-    Copyright (C) 1998-2000 Free Software Foundation, Inc.
-  
+    Copyright (C) 1998-2000, 2007, 2009-2010 Free Software Foundation,
+    Inc.
+
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,8 +14,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <config.h>
@@ -576,7 +576,7 @@ static int ext2_shrink_fs(struct ext2_fs *fs, blk_t newsize,
 			diff -= sizelast;
 		}
 	}
-	
+
 	ped_timer_update (timer, 1.0);
 
 	return 1;
@@ -590,7 +590,7 @@ int ext2_determine_itoffset(struct ext2_fs *fs)
 		       - EXT2_SUPER_FIRST_DATA_BLOCK(fs->sb);
 
         /*PED_DEBUG (0x20, "itoffset is %d", fs->itoffset);
-        
+
         PED_DEBUG (0x20, "walking %d groups", fs->numgroups);*/
 
 	for (i=0;i<fs->numgroups;i++)
@@ -638,7 +638,7 @@ int ext2_determine_itoffset(struct ext2_fs *fs)
                         PED_DEBUG (0x21, "calculated inode table to be %d, "
                                          "but fs says %d.", it,
                                          EXT2_GROUP_INODE_TABLE(fs->gd[i]));*/
-                        
+
 			return 0;
 		}
 	}
@@ -690,11 +690,10 @@ int ext2_resize_fs(struct ext2_fs *fs, blk_t newsize, PedTimer* timer)
                         PED_EXCEPTION_OK_CANCEL,
                         _("A resize operation on this file system will "
                           "use EXPERIMENTAL code\n"
-                          "that MAY CORRUPT it (although it hasn't done"
-                          "so yet\n"
-                          "in the past).\n"
-                          "You should at least backup your data and "
-                          "run 'e2fsck -f' afterwards."))
+                          "that MAY CORRUPT it (although no one has "
+                          "reported any such damage yet).\n"
+                          "You should at least backup your data first, "
+                          "and run 'e2fsck -f' afterwards."))
                 == PED_EXCEPTION_CANCEL)
         {
 	        return 0;
@@ -723,7 +722,7 @@ int ext2_resize_fs(struct ext2_fs *fs, blk_t newsize, PedTimer* timer)
 	else
 		status = ext2_grow_fs(fs, newsize, timer);
 
-	ped_free(fs->relocator_pool);
+	free(fs->relocator_pool);
 	fs->relocator_pool = NULL;
 	fs->relocator_pool_end = NULL;
 
