@@ -1,6 +1,6 @@
 /*
     libparted
-    Copyright (C) 1998, 1999, 2000, 2007 Free Software Foundation, Inc.
+    Copyright (C) 1998-2000, 2007-2010 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -481,8 +481,8 @@ ask_type (PedFileSystem* fs, int fat16_ok, int fat32_ok, FatType* out_fat_type)
 {
 	FatSpecific*		fs_info = FAT_SPECIFIC (fs);
 	PedExceptionOption	status;
-	char*			fat16_msg;
-	char*			fat32_msg;
+	const char*		fat16_msg;
+	const char*		fat32_msg;
 
 	if (fs_info->fat_type == FAT_TYPE_FAT16)
 		fat16_msg = _("If you leave your file system as FAT16, "
@@ -572,7 +572,7 @@ ask_type (PedFileSystem* fs, int fat16_ok, int fat32_ok, FatType* out_fat_type)
 		*out_fat_type = FAT_TYPE_FAT32;
 		return 1;
 	}
-	
+
 	ped_exception_throw (
 		PED_EXCEPTION_NO_FEATURE,
 		PED_EXCEPTION_CANCEL,
@@ -683,7 +683,7 @@ create_resize_context (PedFileSystem* fs, const PedGeometry* new_geom)
 	new_fs_info->cluster_count = new_cluster_count;
 	new_fs_info->dir_entries_per_cluster = fs_info->dir_entries_per_cluster;
 
-	new_fs_info->fat_type = new_fat_type; 
+	new_fs_info->fat_type = new_fat_type;
 	new_fs_info->fat_table_count = 2;
 	new_fs_info->fat_sectors = new_fat_sectors;
 
@@ -711,7 +711,7 @@ create_resize_context (PedFileSystem* fs, const PedGeometry* new_geom)
 			  + 2 * new_fs_info->fat_sectors;
 	} else {
 		new_fs_info->root_dir_sector_count = root_dir_sector_count;
-		new_fs_info->root_dir_entry_count 
+		new_fs_info->root_dir_entry_count
 			= root_dir_sector_count * 512 / sizeof (FatDirEntry);
 
 		new_fs_info->fat_offset
@@ -724,7 +724,7 @@ create_resize_context (PedFileSystem* fs, const PedGeometry* new_geom)
 		new_fs_info->cluster_offset = new_fs_info->root_dir_offset
 					  + new_fs_info->root_dir_sector_count;
 	}
-	
+
 	new_fs_info->total_dir_clusters = fs_info->total_dir_clusters;
 
 	context = fat_op_context_new (new_fs, fs);
@@ -742,11 +742,11 @@ create_resize_context (PedFileSystem* fs, const PedGeometry* new_geom)
 error_free_fat:
 	fat_table_destroy (new_fs_info->fat);
 error_free_context:
-	ped_free (context);
+	free (context);
 error_free_new_fs_info:
-	ped_free (new_fs_info);
+	free (new_fs_info);
 error_free_new_fs:
-	ped_free (new_fs);
+	free (new_fs);
 error:
 	return NULL;
 }
@@ -759,7 +759,7 @@ resize_context_assimilate (FatOpContext* ctx)
 
 	fat_free_buffers (ctx->old_fs);
 	fat_table_destroy (old_fs_info->fat);
-	ped_free (old_fs_info);
+	free (old_fs_info);
 	ped_geometry_destroy (ctx->old_fs->geom);
 
 	ctx->old_fs->type_specific = ctx->new_fs->type_specific;
@@ -768,7 +768,7 @@ resize_context_assimilate (FatOpContext* ctx)
 				? &fat16_type
 			       	: &fat32_type;
 
-	ped_free (ctx->new_fs);
+	free (ctx->new_fs);
 
 	fat_op_context_destroy (ctx);
 
@@ -782,9 +782,9 @@ resize_context_abort (FatOpContext* ctx)
 
 	fat_free_buffers (ctx->new_fs);
 	fat_table_destroy (new_fs_info->fat);
-	ped_free (new_fs_info);
+	free (new_fs_info);
 	ped_geometry_destroy (ctx->new_fs->geom);
-	ped_free (ctx->new_fs);
+	free (ctx->new_fs);
 
 	fat_op_context_destroy (ctx);
 
