@@ -1,10 +1,11 @@
 /*
     parted - a frontend to libparted
-    Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
+    Copyright (C) 1999-2001, 2007, 2009-2010 Free Software Foundation,
+    Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,11 +14,10 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "config.h"
+#include <config.h>
 
 #include <parted/debug.h>
 
@@ -28,6 +28,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <limits.h>
+#include "xalloc.h"
 
 #ifdef ENABLE_NLS
 
@@ -97,7 +98,7 @@ wchar_strdup (const wchar_t* str)
 #ifdef ENABLE_NLS
 	return wcsdup (str);
 #else
-	return strdup (str);
+	return xstrdup (str);
 #endif
 }
 
@@ -123,12 +124,12 @@ gettext_to_wchar (const char* str)
 	if (status == (size_t) -1)
 		goto error;
 
-	result = realloc (result, (wcslen (result) + 1) * sizeof (wchar_t));
+	result = xrealloc (result, (wcslen (result) + 1) * sizeof (wchar_t));
 	return result;
 
 error:
 	printf ("Error during translation: %s\n", strerror (errno));
-	exit (1);
+	exit (EXIT_FAILURE);
 }
 
 #else /* ENABLE_NLS */
@@ -136,7 +137,7 @@ error:
 static wchar_t*
 gettext_to_wchar (const char* str)
 {
-	return strdup (str);
+	return xstrdup (str);
 }
 
 #endif /* !ENABLE_NLS */
@@ -177,7 +178,7 @@ wchar_to_str (const wchar_t* str, size_t count)
 
 error:
 	printf ("Error during translation: %s\n", strerror (errno));
-	exit (1);
+	exit (EXIT_FAILURE);
 }
 
 #else /* ENABLE_NLS */
@@ -187,7 +188,7 @@ wchar_to_str (const wchar_t* str, size_t count)
 {
 	char*		result;
 
-	result = strdup (str);
+	result = xstrdup (str);
 	if (count && count < strlen (result))
 		result [count] = 0;
 	return result;
@@ -203,12 +204,12 @@ print_wchar (const wchar_t* str, size_t count)
 	free (tmp);
 }
 
-static StrList* 
+static StrList*
 str_list_alloc ()
 {
 	StrList*	list;
 
-	list = (StrList*) malloc (sizeof (StrList));
+	list = xmalloc (sizeof (StrList));
 	list->next = NULL;
 
 	return list;
@@ -226,7 +227,8 @@ str_list_destroy (StrList* list)
 void
 str_list_destroy_node (StrList* list)
 {
-	free ((wchar_t*) list->str);
+	void *p = (char *) (list->str); /* discard const */
+	free (p);
 	free (list);
 }
 
@@ -360,7 +362,7 @@ str_list_convert (const StrList* list)
 	const StrList*	walk;
 	int		pos = 0;
 	int		length = 1;
-	char*		str = strdup ("");
+	char*		str = xstrdup ("");
 
 	for (walk = list; walk; walk = walk->next) {
 		if (walk->str) {
@@ -390,20 +392,6 @@ str_list_print (const StrList* list)
 	}
 }
 
-static char*
-get_spaces (int space_count)
-{
-	char*	str;
-	int	i;
-
-	str = malloc (space_count + 1);
-	for (i = 0; i < space_count; i++)
-		str [i] = ' ';
-	str [i] = 0;
-
-	return str;
-}
-
 static int
 str_search (const wchar_t* str, int n, wchar_t c)
 {
@@ -459,13 +447,11 @@ str_list_print_wrap (const StrList* list, int line_length, int offset,
 	int		cut_right;
 	int		cut_left;
 	int		line_left;
-	char*		spaces;
 	int		search_result;
 	int		line_break;
 
 	PED_ASSERT (line_length - indent > 10, return);
 
-	spaces = get_spaces (indent);
 	line_left = line_length - offset;
 
 	for (walk=list; walk; walk=walk->next) {
@@ -511,16 +497,14 @@ str_list_print_wrap (const StrList* list, int line_length, int offset,
 			line_left = line_length - indent;
 
 			if (walk->next || *str)
-				printf ("\n%s", spaces);
+				printf ("\n%*s", indent, "");
 			else if (line_break)
-				printf ("\n");
+				putchar ('\n');
 		}
 
 		print_wchar (str, 0);
 		line_left -= wchar_strlen (str);
 	}
-
-	free (spaces);
 }
 
 static int
@@ -602,4 +586,3 @@ str_list_length (const StrList* list)
 
 	return length;
 }
-
