@@ -1,6 +1,6 @@
 /*
     parted - a frontend to libparted
-    Copyright (C) 1999-2002, 2006-2009 Free Software Foundation, Inc.
+    Copyright (C) 1999-2002, 2006-2010 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -193,6 +193,11 @@ static StrList*     ex_opt_str [64];
 static StrList*     on_list;
 static StrList*     off_list;
 static StrList*     on_off_list;
+
+static StrList*     align_opt_list;
+static StrList*     align_min_list;
+static StrList*     align_opt_min_list;
+
 static StrList*     fs_type_list;
 static StrList*     disk_type_list;
 
@@ -314,7 +319,7 @@ reset_env (int quit)
         if (in_readline) {
                 putchar ('\n');
                 if (quit)
-                        exit (0);
+                        exit (EXIT_SUCCESS);
 
                 siglongjmp (readline_state.jmp_state, 1);
         }
@@ -1259,6 +1264,27 @@ command_line_get_ex_opt (const char* prompt, PedExceptionOption options)
         return opt;
 }
 
+int
+command_line_get_align_type (const char *prompt, enum AlignmentType *align_type)
+{
+  char*    def_word;
+  char*    input;
+
+  if (*align_type)
+    def_word = str_list_convert_node (align_opt_list);
+  else
+    def_word = str_list_convert_node (align_min_list);
+  input = command_line_get_word (prompt, def_word, align_opt_min_list, 1);
+  free (def_word);
+  if (!input)
+    return 0;
+  *align_type = (str_list_match_any (align_opt_list, input)
+	     ? PA_OPTIMUM
+	     : PA_MINIMUM);
+  free (input);
+  return 1;
+}
+
 int
 command_line_get_unit (const char* prompt, PedUnit* unit)
 {
@@ -1346,6 +1372,24 @@ done_state_str ()
         str_list_destroy (on_off_list);
 }
 
+static int
+init_alignment_type_str ()
+{
+        align_opt_list = str_list_create_unique (_("optimal"), "optimal", NULL);
+        align_min_list = str_list_create_unique (_("minimal"), "minimal", NULL);
+        align_opt_min_list = str_list_join (str_list_duplicate (align_opt_list),
+					    str_list_duplicate (align_min_list));
+        return 1;
+}
+
+static void
+done_alignment_type_str ()
+{
+        str_list_destroy (align_opt_list);
+        str_list_destroy (align_min_list);
+        str_list_destroy (align_opt_min_list);
+}
+
 static int
 init_fs_type_str ()
 {
@@ -1409,6 +1453,7 @@ init_ui ()
 {
         if (!init_ex_opt_str ()
             || !init_state_str ()
+            || !init_alignment_type_str ()
             || !init_fs_type_str ()
             || !init_disk_type_str ())
                 return 0;
@@ -1453,6 +1498,7 @@ done_ui ()
         ped_exception_set_handler (NULL);
         done_ex_opt_str ();
         done_state_str ();
+        done_alignment_type_str ();
         str_list_destroy (fs_type_list);
         str_list_destroy (disk_type_list);
 }
@@ -1471,7 +1517,7 @@ help_msg ()
         fputs (_("COMMANDs:"), stdout);
         putchar ('\n');
         print_commands_help ();
-        exit (0);
+        exit (EXIT_SUCCESS);
 }
 
 void
@@ -1553,7 +1599,7 @@ non_interactive_mode (PedDevice** dev, Command* cmd_list[],
                 if (!(cmd->non_interactive)) {
                         fputs(_("This command does not make sense in "
                                 "non-interactive mode.\n"), stdout);
-                        exit(1);
+                        exit(EXIT_FAILURE);
                         goto error;
                 }
 
